require 'date'
require 'chronic'

require 'middleman-core/cli'

module Middleman
  module Cli
    ##
    # This class provides an "entry" command for the middleman CLI.
    #
    # @usage bundle exec middleman entry --help
    # @usage bundle exec middleman entry
    # @usage bundle exec middleman entry -d yesterday
    #
    ##
    class Entry < ::Thor::Group
      include Thor::Actions

      check_unknown_options!

      # Template files are relative to this file
      # @return [String]
      def self.source_root
        File.dirname(__FILE__)
      end

      class_option 'date',
                   aliases: '-d',
                   desc: 'Date of the entry (default: today)'

      def entry
        @date = options[:date] ? Chronic.parse(options[:date]) : Date.today
        @title = @date.strftime('%F')
        @slug = @date.strftime('%F')

        template template_path, destination_path
      end

      private

      def template_path
        File.expand_path('entry.tt', File.dirname(__FILE__))
      end

      def destination_path
        app = ::Middleman::Application.new do
          config[:mode]              = :config
          config[:disable_sitemap]   = true
          config[:watcher_disable]   = true
          config[:exit_before_ready] = true
        end

        path = File.join(app.source_dir, 'journal', "#{@title}.html.markdown")

        if File.exist?(path)
          throw "An entry for #{@title} already exists: #{path}"
        end

        path
      end

      # Add to CLI
      Base.register(self, 'entry', 'entry', 'Add a new entry to the journal')
    end
  end
end
